@@ -1,11 +1,11 @@
 from pyais.messages import NMEAMessage, ANY_MESSAGE, AISSentence
-from pyais.stream import TCPConnection, FileReaderStream, IterMessages
+from pyais.stream import TCPConnection, FileReaderStream, IterMessages, Stream, PreprocessorProtocol
 from pyais.encode import encode_dict, encode_msg, ais_to_nmea_0183
 from pyais.decode import decode
 from pyais.tracker import AISTracker, AISTrack
 
 __license__ = 'MIT'
-__version__ = '2.5.6'
+__version__ = '2.8.1'
 __author__ = 'Leon Morten Richter'
 
 __all__ = (
@@ -18,6 +18,8 @@ __all__ = (
     'TCPConnection',
     'IterMessages',
     'FileReaderStream',
+    'Stream',
+    'PreprocessorProtocol',
     'decode',
     'AISTracker',
     'AISTrack',
