@@ -155,6 +155,52 @@ def error_if_uninitialized(func: typing.Callable[['TagBlock'], typing.Any]) -> t
     return wrapper
 
 
+class TagBlockGroup:
+    """Tag Block Group represents the 3-int group sequence
+    optionally included as part of the NMEA Tag Block
+
+    it consists of 3, comma-seperated integers X-Y-Z where:
+    - X = Message ID in sequence
+    - Y = Total parts in group
+    - Z = Unique GroupID for this group of messages."""
+
+    __slots__ = (
+        'sentence_num',
+        'sentence_tot',
+        'group_id'
+    )
+
+    def __init__(self, msg_id: int, total: int, group_id: int):
+        self.sentence_num = msg_id
+        self.sentence_tot = total
+        self.group_id = group_id
+
+    @staticmethod
+    def from_str(raw: str) -> 'TagBlockGroup':
+        """Constructs a new NMEAGroup from it's string representation"""
+        [msg_id, msg_total, group_id] = raw.split("-", 3)
+
+        return TagBlockGroup(
+            int(msg_id),
+            int(msg_total),
+            int(group_id)
+        )
+
+    @property
+    def is_fragmented(self) -> bool:
+        """Returns whether or not this group expects several parts."""
+        return self.sentence_tot > 1
+
+    def __str__(self) -> str:
+        """Returns this NMEA group instance in it's string representation."""
+        return f"{self.sentence_num}-{self.sentence_tot}-{self.group_id}"
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, TagBlockGroup):
+            return self.sentence_num == other.sentence_num and self.sentence_tot == other.sentence_tot and self.group_id == other.group_id
+        return False
+
+
 class TagBlock:
 
     __slots__ = (
@@ -169,6 +215,7 @@ class TagBlock:
         '_line_count',
         '_relative_time',
         '_text',
+        '_group'
     )
 
     def __init__(self, raw: bytes) -> None:
@@ -183,6 +230,7 @@ class TagBlock:
         self._source_station: typing.Optional[str] = None
         self._relative_time: typing.Optional[str] = None
         self._text: typing.Optional[str] = None
+        self._group: typing.Optional[TagBlockGroup] = None
 
     @property
     @error_if_uninitialized
@@ -229,6 +277,11 @@ class TagBlock:
     def expected_checksum(self) -> int:
         return self._expected_checksum
 
+    @property
+    @error_if_uninitialized
+    def group(self) -> typing.Optional[TagBlockGroup]:
+        return self._group
+
     def init(self) -> None:
         payload, check = self.raw.split(ASTERISK)
 
@@ -257,6 +310,8 @@ class TagBlock:
                 self._source_station = val
             elif spec == 't':
                 self._text = val
+            elif spec == 'g':
+                self._group = TagBlockGroup.from_str(val)
             else:
                 pass
 
@@ -573,7 +628,7 @@ class Payload(abc.ABC):
 
         d_type = field.metadata['d_type']
 
-        if type(val) == d_type:
+        if isinstance(val, d_type):
             # The value is already of the correct type -> nothing to do
             return val
 
@@ -777,7 +832,7 @@ def to_turn(turn: typing.Union[int, float]) -> typing.Union[float, TurnRate]:
     elif abs(turn) == 128:
         return TurnRate.NO_TI_DEFAULT
 
-    return math.copysign(int((turn / 4.733) ** 2), turn)
+    return math.copysign(round((turn / 4.733) ** 2), turn)
 
 
 def from_turn(turn: typing.Optional[typing.Union[int, float, TurnRate]]) -> int:
@@ -797,10 +852,14 @@ class CommunicationStateMixin:
     https://www.itu.int/dms_pubrec/itu-r/rec/m/R-REC-M.1371-1-200108-S!!PDF-E.pdf
     """
 
+    msg_type: int  # Type hint to make mypy happy
     radio: int  # Type hint to make mypy happy
 
     MAX_COMM_STATE_VALUE = 0x7ffff
 
+    SOTDMA_TYPES = (1, 2, 4, 11)
+    SOTDMA_ITDMA_TYPES = (9, 18, 26)
+
     def get_communication_state(self) -> Dict[str, typing.Optional[int]]:
         """Returns information used by the slot allocation algorithm as a dict."""
         result: Dict[str, typing.Optional[int]] = {
@@ -825,13 +884,21 @@ class CommunicationStateMixin:
 
     @property
     def is_sotdma(self) -> bool:
-        """The radio status field has it's 20th bit (MSB) set to 0 or has less than 20 bits"""
-        return self.radio <= self.MAX_COMM_STATE_VALUE
+        """Messages of type 1, 2, 4, 11 use SOTDMA or 9, 18, 26 if 20th bit is set."""
+        if self.msg_type in self.SOTDMA_TYPES:
+            return True
+        if self.msg_type in self.SOTDMA_ITDMA_TYPES:
+            return self.radio <= self.MAX_COMM_STATE_VALUE
+        return False
 
     @property
     def is_itdma(self) -> bool:
-        """The radio status field has it's 20th bit (MSB) set to 1"""
-        return self.radio > self.MAX_COMM_STATE_VALUE
+        """Messages of type 3 use ITDMA or 9, 18, 26 if 20th bit is set."""
+        if self.msg_type == 3:
+            return True
+        if self.msg_type in self.SOTDMA_ITDMA_TYPES:
+            return self.radio > self.MAX_COMM_STATE_VALUE
+        return False
 
     @property
     def communication_state_raw(self) -> int:
